#!/usr/bin/python

# ===================================================================
# @(#)mrt.py
#
# @author Virginie Van den Schrieck
#               (virginie.vandenschrieck@uclouvain.be)
# @date  1/10/2008
# @lastdate 01/11/08
# ===================================================================

# IMPORTS
import sys
import os


#
# This function returns true if line is a MRT formatted BGP route, else otherwise
#
def is_route(line) : 
    if line=="" : 
        return False
    if not (len(line.split('|'))==15 or len(line.split('|'))==13 or len(line.split('|'))==6) : 
        return False
    if not "TABLE_DUMP" in line and not "BGP4" in line and not "BGP4MP "in line : 
        return False
    return True

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the prefix annonced in the BGP route line
#
def get_prefix(line) : 
    return line.split('|')[5]

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the loc pref of the BGP route line
#
def get_locpref(line) : 
    if line.split('|')[9]=="" : 
        return 0
    return int(line.split('|')[9])

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the MED of the BGP route line
#
def get_med(line) : 
    if line.split('|')[10]=="" : 
        return 0
    return int(line.split('|')[10])

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the length of the aspath of the BGP route line
#
def get_aspath_length(line) : 
    aspath=line.split('|')[6]
    return len(aspath.split())

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the list of ASes in the AS path of this BGP route
#
def get_aspath(line) : 
    aspath=line.split('|')[6]
    return aspath.split()

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the nexthop of the BGP route line
#
def get_nexthop(line) : 
    return line.split('|')[8]

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the timestamp of the BGP route line
#
def get_timestamp(line) : 
    return line.split('|')[1]

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the IP of the peer that advertised the BGP route 
# line
#
def get_peer_ip(line) : 
    return line.split('|')[3]

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the AS number of the peer that advertised the 
# BGP route line
#
def get_peer_as(line) : 
    return line.split('|')[4]

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the origin flag the 
# BGP route line
#
def get_flag(line) : 
    return line.split('|')[7]

#
# PRE : Line is a MRT formatted BGP Route
# This function returns the communities of the 
# BGP route line as a string
#
def get_communities(line) : 
    return line.split('|')[-1]

#
# PRE : Line is a MRT formatted BGP Route
# This function returns type of the route (A, W or STATE)
#
def get_route_type(line) : 
    return line.split('|')[2]
