#!/usr/bin/env python

################################################################################
# select_top_ASes.py                                                           #
#                                                                              #
# Author Yves Vanaubel (yves.vanaubel@ulg.ac.be)                               #
# Date 08/01/15                                                                #
# Description: Select the top ASes with the more MPLS IPs (cumulative)         #
#              If nb_top_ases is 0, all ASes are selected                      #
################################################################################

# IMPORTS
import sys
import os
import re
import numpy as np
import subprocess
from optparse import OptionParser

def main():
	usage = "usage: %prog <output_dir> <cycle_filtered_tunnels_file> <MPLS_array_file> <nb_top_ases>"
	
	parser = OptionParser(usage=usage)
	
	(options, args) = parser.parse_args()
    
	#Check arg number
	if not len(args) == 4:
	   parser.print_help()
	   sys.exit(1)
	run(args)

################################################################################

def run(args):
    
    output_dir = args[0]
    cycle_filtered_tunnels_file = args[1]
    mpls_array_file = args[2]
    nb_top_ases = args[3]
    
    # Read the AS array
    asn2nb_mpls_ips = dict()
    os.system('grep -v -e "#" -e "NA" ' + mpls_array_file + ' > tmp_as_selection_array.txt')
    as_array_file = open('tmp_as_selection_array.txt')
        
    # Read the array
    for as_line in as_array_file:
        # Skip empty lines
        if as_line == "":
            continue
    
        line_split = as_line.split()
        asn = line_split[0]
        nb_mpls_ips = line_split[1]
        if asn in asn2nb_mpls_ips:
            asn2nb_mpls_ips[asn] += int(nb_mpls_ips)
        else:
            asn2nb_mpls_ips[asn] = int(nb_mpls_ips)
    os.system('rm tmp_as_selection_array.txt')
    
    # Select the top X ASes
    ases = list()
    nb_selected_as = 0
    for top_as in sorted(asn2nb_mpls_ips.items(), key=lambda x:x[1], reverse=True):
        if top_as[1] == 0:
            break
        ases.append(top_as[0])
        nb_selected_as += 1
        if nb_top_ases != 0 and nb_selected_as == nb_top_ases:
            break

    os.system('rm -rf ' + output_dir)
    os.system('mkdir ' + output_dir)

    # Select the tunnels for each AS
    for asn in ases:
        os.system("awk -v asn=" + str(asn) + " '{if ($1 == asn) {$1 = \"\"; gsub(/^ /, \"\", $0); print $0}}' " + cycle_filtered_tunnels_file + " > " + output_dir + "/as_" + str(asn) + ".txt")

    top_as_output_file = open(output_dir + '/top_ases.txt', 'w')
    for asn in ases:
        top_as_output_file.write(asn + '\n')
    top_as_output_file.close()

    print "Tunnels extracted!"

if __name__ == '__main__':
	sys.exit(main())
