#!/usr/bin/env python

################################################################################
# clean_caida_data.py                                                          #
#                                                                              #
# Author Yves Vanaubel (yves.vanaubel@ulg.ac.be)                               #
# Date 25/11/14                                                                #
# Description: Clean the traces of CAIDA :                                     #
#               1. Deletes multiple RTTs                                       #
#               2. Deletes traces with multiple IPs for a given hop            #
################################################################################

# IMPORTS
import sys, re
import numpy as np
from optparse import OptionParser

def main():
	usage = "usage: %prog <input_file> <output_file>"
	
	parser = OptionParser(usage=usage)
	
	(options, args) = parser.parse_args()
    
	#Check arg number
	if not len(args) == 2:
	   parser.print_help()
	   sys.exit(1)
	run(args)

# Write a trace in the output file
def print_trace(lines, output_file):
    for l in lines:
        output_file.write(l + '\n')


################################################################################

def run(args):
    
    input_file = args[0]
    output_file = args[1]
    
    traces_file = open(input_file)
    clean_file = open(output_file, 'w')
        
    trace_lines = []
    accepted = True
    
    nb_deleted_traces = 0
    nb_traces = 0
    
    # Get each trace in the file
    for line in traces_file:
        line = line.strip()
        # Skip empty lines
        if line == "":
            continue
        
        # Check if new trace
        if line.startswith("traceroute"):
            nb_traces += 1
            # The trace is accepted, and must be printed
            if accepted and trace_lines:
                print_trace(trace_lines, clean_file)
            # Accept by default
            accepted = True
            # Clean data structure and add new trace
            del trace_lines[:]
            trace_lines.append(line)
            continue
        
        # Node of a trace
        
        # Bad node
        if line.count("(") > 1:
            nb_deleted_traces += 1
            accepted = False
            continue
                
        # Multiple probes
        if line.count("ms") > 1:
            trace_lines.append(re.sub(r"\).* ( .* ms( +![^ ]+)*)", r") \1", line))
            continue

        trace_lines.append(line)
        
    # Last trace
    if accepted and trace_lines:
        print_trace(trace_lines, clean_file)

    # Print on the screen some stats
    print nb_deleted_traces

    clean_file.close()

if __name__ == '__main__':
	sys.exit(main())
