#!/usr/bin/env python

################################################################################
#   Author : Yves Vanaubel                                                     #
#   Date : 19/06/2014                                                          #
#                                                                              #
#   Map IPs to ASn                                                             #
#   The input file is a list of IPs                                            #
################################################################################

# IMPORTS
import sys
import numpy as np
import subprocess
import itertools as it
from optparse import OptionParser

def main():
    usage = "usage: %prog <prefix2asn_file> <input_file> <output_file>"

    parser = OptionParser(usage=usage)

    (options, args) = parser.parse_args()

    #Check arg number
    if not len(args) == 3:
        parser.print_help()
        sys.exit(1)
    run(args)

################################################################################

def run(args):

    prefix2asn_file = args[0]
    input_file = args[1]
    output_file = args[2]

    # Read the AS numbers
    asn = dict()
    multilabeled = []
    prefix2asn_file = open(prefix2asn_file)

    # Get each tunnel in the file
    for prefix2asn in prefix2asn_file:
        prefix2asn = prefix2asn.strip()
        # Skip empty lines
        if prefix2asn == "":
            continue

        prefix2asn_array = prefix2asn.split()
        prefix = prefix2asn_array[0]
        if prefix in asn and prefix not in multilabeled:
            multilabeled.append(prefix)
        else:
            asn[prefix] = prefix2asn_array[1]
    prefix2asn_file.close()

    # Delete the multilabeled ASes
    for multi in multilabeled:
        asn[multi] = "MO"

    output_file = open(output_file, 'w')
    input_file = open(input_file)

    # Read the tunnels
    for line in input_file:
        line = line.strip()
        # Skip empty lines
        if line == "":
            continue
        
        ip = line

        # Transform the IP address into a /24 prefix
        ip_split = ip.split('.')
        p = ip_split[0] + '.' + ip_split[1] + '.' + ip_split[2] + '.0'

        # Get the ASn if any
        if p in asn:
            output_file.write(line + ' ' + asn[p] + '\n')
        else:
            output_file.write(line + ' NA\n')

    output_file.close()

if __name__ == '__main__':
    sys.exit(main())
