#!/bin/bash

################################################################################
#   Author : Yves Vanaubel                                                     #
#   Date : 09/01/2015                                                          #
#                                                                              #
#   Get the ASn associated to the IP addresses for a cycle                     #
#    - cycle_dir : The directory containing the VPs                            #
#    - output dir : a directory containing files of pairs <prefix asn>         #
#    - compress_results : 1 if the results must be compressed                  #
#       Note : The reference time is 00:01:00                                  #
################################################################################

shopt -s extglob

if [ $# -ne 3 ]
then
    echo 'Usage : ./get_cycle_prefix2as_mapping.sh <cycle_warts_dir> <output_dir> <compress_results>'
    exit 1
fi

cycle_dir=$1
output_dir=$2
must_compress=$3

if [ -e $output_dir ]
then
    rm -rf $output_dir
fi
mkdir $output_dir
mkdir $output_dir/computing_dir
computing_dir=$output_dir/computing_dir

script_dir=$(dirname $0)
root_script_dir=$script_dir/../

counter=0
# Analyze the warts file for each VP
for d in `ls $cycle_dir`;
do
    let 'counter = counter + 1'
    if test -d $cycle_dir/$d
    then
        ptmp=$cycle_dir/$d
        if [ -e $ptmp/warts ]
        then
            ptmp=$ptmp/warts
            let 'j = 0'
            # Analyzes the warts files.
            for w in `ls $ptmp/*.warts.gz`;
            do
                # Unzip the warts
                unz_w=$(basename $w .gz)
                gunzip -c $w > $computing_dir/$unz_w

                # sc_mpls_anal
                error=1
                sc_warts2text $computing_dir/$unz_w > $computing_dir/"$d"_"$j" 2> $computing_dir/stderr.txt
                error=$?
                rm -rf $computing_dir/$unz_w

                # Check if any error
                grep -v "truncated" $computing_dir/stderr.txt > $computing_dir/stderr2.txt
                mv $computing_dir/stderr2.txt $computing_dir/stderr.txt
                if [ -s $computing_dir/stderr.txt ]
                then
                    echo "Error for $d :"
                    cat $computing_dir/stderr.txt
                    error=1
                fi
                # Delete the last traceroute if needed (may be corrupted, incomplete)
                if [ $error -ne 0 ]
                then
                    trunc=$(cat -n $computing_dir/"$d"_"$j" | grep "traceroute" | tail -n 1 | awk '{print $1}')
                    let 'trunc = trunc - 1'
                    if [ $trunc -gt 0 ]
                    then
                        head -n $trunc $computing_dir/"$d"_"$j" > $computing_dir/updated
                        mv $computing_dir/updated $computing_dir/"$d"_"$j"
                    else
                        # The file seems completely corrupted
                        echo -n "" > $computing_dir/"$d"_"$j"
                    fi
                fi
                let 'j = j + 1'
            done

            cd $computing_dir/
            cat "$d"_* > $d
            rm -f !($d)
            cd - > /dev/null 2>&1

            # Cleaning traces
            tmp_nb=$(python $root_script_dir/cleaning/clean_caida_traces.py $computing_dir/$d $computing_dir/clean)
            mv $computing_dir/clean $computing_dir/$d

            # Extracts the IP addresses from the data.
            perl $script_dir/read_warts_ips.pl $computing_dir/$d $computing_dir/ips.txt
            cat $computing_dir/ips.txt >> $output_dir/all_ips.txt
            rm $computing_dir/ips.txt $computing_dir/$d
        fi
    fi
echo -n -e "\rVP $counter"
done

# Sort the IP addresses uniquely
sort -u -o $output_dir/all_ips.txt $output_dir/all_ips.txt

# Get the corresponding date
cur_date=$(basename $cycle_dir)

# Map the IP addresses to ASn
echo "Mapping prefix to AS..."
$script_dir/map_prefix2asn_routeviews.sh $output_dir/all_ips.txt $output_dir/prefix2as.txt $output_dir/moas.txt $cur_date
echo "Done"

rm $output_dir/all_ips.txt

# Compress the results if needed
if [ $must_compress -eq 1 ]
then
    cd $output_dir
    gzip prefix2as.txt
    gzip moas.txt
    cd - > /dev/null 2>&1
fi

rm -r $computing_dir
echo

exit 0
