#!/usr/bin/env python

# ===================================================================
# @(#)make_mapping_from_mrinfo_output.py
#
# @author Virginie Van den Schrieck
#				(virginie.vandenschrieck@uclouvain.be)
#         Modified by Yves Vanaubel on 19/03/2015
# @date 22/ 3/2009
# @lastdate 25/03/09
# ===================================================================

# IMPORTS
import sys
import os
import radix
import mrt
from optparse import OptionParser
DEFAULT_ARGS=True

def main():
	usage = "usage: %prog <prefixes> <rib file> <output MOAS file>"
	
	parser = OptionParser(usage=usage)
	
	(options, args) = parser.parse_args()
	
	#Check arg number
	if not len(args)== 3:
	   parser.print_help()
	   sys.exit(1)
	run(args)

###############################################################################

def run(args):
	num_ip=0
	#1. Collect IP adresses
	ip_list=set()
	file=open(args[0])
	for line in file :
		line=line.strip()
	   #Skip empty lines
	   	if line=="" :
			continue
		num_ip+=1
		if line not in ip_list :
			ip_list.add(line)
	
	file.close()
	
	#2. Build radix tree from MRT rib
	as_set=0
	num_pref_radix_tree=0
	rtree=radix.Radix()
	prefix=""
	file=open(args[1])
	for line in file :
		line=line.strip() 
		if not mrt.is_route(line) :
			continue
		prefix=mrt.get_prefix(line)
		if prefix == '0.0.0.0/0':
			continue
		num_pref_radix_tree+=1
		#Add prefix to radix tree if needed
		node = rtree.search_exact(prefix)
		if node ==None :
			node=rtree.add(prefix)
			node.data["origin"]=[]
		#Insert data into node
		origin=mrt.get_aspath(line)[-1]
		#Origin AS is AS set :
		if '}' in origin :
			as_set+=1
		   #Traverse ASPath from the end
			aspath=mrt.get_aspath(line)
			aspath.reverse()	
			end_set=False
			for asn in aspath :
				if end_set :
					break
			  #First asn in the set : remove parenthesis and stop computation.
				if '{' in asn :
					asn=asn[1:]
					end_set=True
			  #Last asn in the set : remove parenthesis
				if '}' in asn :
					asn=asn[:-1]
				#Add asn to node data
				if asn not in node.data["origin"] :
					node.data["origin"].append(asn)
					node.data["origin"].sort()
		#Normal case : no AS Set
		else :
			if origin not in node.data["origin"] :
				node.data["origin"].append(origin)
				node.data["origin"].sort()
	file.close()
	
	#3. For each IP, check for origin AS and print on stdout
	
	total_ip=0
	pref_not_found=0
	moas_file=open(args[2], 'w')
	#Iterate on IP list
	for ip in ip_list :
		total_ip+=1
		try :
			node=rtree.search_best(ip)
		except ValueError :
		   #Line is not a valid key
			sys.stderr.write("ValueError with line : "+ip+"\n")
			sys.stderr.write("Aborting\n")
			sys.exit(-1)
		#IP not found, print only ip without as list
		if node ==None :
			pref_not_found+=1
			print ip, "NA"
		#Regular case : one or more origin as stored in radix tree
		if node!=None :
			if len(node.data["origin"])>1 :
				moas_file.write(ip)
				for origin in node.data["origin"] :
					moas_file.write(" " + origin)
				moas_file.write("\n")
               			sys.stdout.write(ip + " NA\n")
			else :
				selected_origin = node.data["origin"][0]
				as_nbs = selected_origin.split(",")
				if len(as_nbs) > 1 :
					moas_file.write(ip + " " + selected_origin + "\n")
                    			sys.stdout.write(ip + " NA\n")
					continue
				selected_origin = as_nbs[0]
				sys.stdout.write(ip + " " + selected_origin + "\n")

if __name__ == '__main__':
	sys.exit(main())
