#!/bin/bash

################################################################################
#   Author : Yves Vanaubel                                                     #
#   Date : 19/06/2014                                                          #
#                                                                              #
#   Get the ASn associated to a set of IPs:                                    #
#    - input file : a list of IP addresses to map                              #
#    - output file : a list of pairs <prefix asn>                              #
#    - date : the day when the measures were taken (format: yyyy-mm-dd)        #
#       Note : The reference time is 00:01:00                                  #
################################################################################

if [ $# -ne 4 ]
then
    echo 'Usage : ./map_prefix2asn_routeviews.sh <ips_file> <output_file> <output_moas_file> <date>'
    exit 1
fi

input_file=$1
output_file=$2
output_moas_file=$3
date=$4

script_dir=$(dirname $0)

# Get the /24 prefix for IP addresses
awk '{split($1, ip, "."); print ip[1]"."ip[2]"."ip[3]".0"}' $input_file | sort -u > prefixes

# Parse the date
year=`echo $date | cut -d '-' -f3`
month=`echo $date | cut -d '-' -f2`
day=`echo $date | cut -d '-' -f1`

# Get the prefix2asn mapping
wget -q http://archive.routeviews.org/bgpdata/$year.$month/RIBS
rib=`grep "$year$month$day" RIBS | head -n 1 | cut -d '<' -f7 | cut -d '>' -f2`
wget -q http://archive.routeviews.org/bgpdata/$year.$month/RIBS/$rib
bgpdump -m $rib > DUMP
python $script_dir/map_prefix2asn.py prefixes DUMP $output_moas_file | sort -k2n,2n > $output_file
rm -vf RIBS
rm -vf $rib
rm -vf DUMP
rm -vf prefixes

exit 0
