#!/usr/bin/perl

################################################################################
#   Author : Yves Vanaubel on 13/06/2014                                       #
#                                                                              #
#   Extract the IP addresses seen by the VP                                    #
################################################################################


# Include the modules.
use File::Find;
use File::Basename;
use POSIX qw(strftime);
use strict;

# Check the arguments.
if ($#ARGV != 1)
{
	print "Usage: read_warts_ips.pl <traces_file> <output_file> \n";
    exit;
}

# Gets the arguments.
my($inputFile) = $ARGV[0];
my($outputFile) = $ARGV[1];

open(my $input_file, "<", $inputFile) || die ("Error when opening $inputFile");
open(my $ips_file, ">", "$outputFile") || die ("Error when opening the ip's file.\n");

my($line, $target, $ipAddress);

my(%privateIps) = ();            # Private IP addresses

# Read the file.

# Example of format : 
#
# traceroute from 195.148.124.66 to 12.100.180.192
# 12  (12.122.1.2)  125.662 ms !T2 MPLS Label 0 TTL=255 MPLS Label 22927 TTL=255


while ($line = <$input_file>)
{
    # Analyze the current line.
    chop($line);
    
    # Skip empty lines.
    if ($line eq "")
    {
        next;
    }
    
    # Get the destination of the traceroute
    if ($line =~ /^traceroute/)
    {
        $target = $line;
        $target =~ s/.*to //;        # Delete all the characters before the last 'to '.
        
        print $ips_file "$target\n";
        next;
    }
    
    if ($line =~ /^[ ]*[0-9][0-9]* / )
    {
        # Get the IP address.
        my($original_line) = $line;
        $line =~ s/^[ 0-9]*[0-9][ ]+//; # Delete the hop number
        $ipAddress = $line;
        $ipAddress =~ s/^\(//;          # Delete all the characters before the last '('.
        $ipAddress =~ s/\).*//;         # Delete all the character after the last ')'.

        # Check if it is a private IP address
        if (($original_line =~ /rfc1918$/) || ($original_line =~ /local$/))
        {
            $privateIps{$ipAddress} = "T";
        }
        
        # Register the IP address
        if ($ipAddress ne "*")
        {
            if (exists $privateIps{$ipAddress})
            {
                next;
            }
            else
            {
                print $ips_file "$ipAddress\n";
            }
        }
        next;
    }
}


# Close the files.
close($ips_file);
close($input_file);

` sort -u -o  $outputFile $outputFile ` ;
